/* Folder namespace */
impro.editor = {};

/* init the editor support */
impro.editor.init = function() {

	/* set a reference to the image editor */
	impro.editor.$editor = jQuery('#impro_editor');
	
	/* start with the editor closed */
	jQuery('#impro_editor_box').addClass('closed').find('.handlediv').hide();
		
	/* init events */
	impro.editor.initEvents();
	
	/* if the visual editor is enabled in the user preferences */
	if (jQuery('#edButtonPreview').size() > 0 || jQuery('#content-tmce').size() > 0) {
		/* hook up the switchEditors to find out when the editor is changed */
		/* TODO: This does not work for WordPress 3.2 */
		try {
			switchEditors.old_go = switchEditors.go;
			switchEditors.go = function(id, mode) {
				switchEditors.old_go(id, mode);
				impro.editor.onSwitchEditor(id, mode);
			};
		} catch(ex) {}
		
		/* tinyMCE will start initialization soon */
		var tinymce_timer = setInterval(function() {
			if (window.tinymce && window.tinymce.getInstanceById('content')) {
				clearInterval(tinymce_timer);
				
				/* prevent the user opening the image settings panel */
				jQuery('#impro_editor_box').find('.hndle').unbind('click');
				
				/* initialize the editor */
				impro.editor.onEditorInit(window.tinymce.getInstanceById('content'));
			}
		}, 300);
	} else {
		impro.err(imageproEditorL10N.enableTinyMce);
	}
};

/* get TinyMCE */
impro.editor.getTinyMCE = function() {
	return tinymce.getInstanceById('content');
};

/* fires when tinymce is initialized */
impro.editor.onEditorInit = function(editor) {
	/* now waiting for the iframe inside to render the post and have the body */
	var timer = setInterval(function() {
		var body = jQuery(window.tinymce.getInstanceById('content').contentAreaContainer).find('iframe').contents().find('body');
		if (body.size() > 0) {
			clearInterval(timer);
			
			/* link the click event to tiny mce */
			impro.editor.setTinyMCEInteractions(body, window.tinymce.getInstanceById('content'));
			
			/* if we're on Chrome or Safari, try to simulate the resize handles */
			if (window.tinymce.isWebKit) {
				impro.editor.fakeImageResize();
			}
		}
	}, 300);
};

/* set the onclick event on the editor */
impro.editor.setTinyMCEInteractions = function(body, editor) {	
	
	/* register drop event to catch drops */
	setInterval(function() {
		/* because the ondrop event is not at all reliable
		 * not to say about it's cross browser support,
		 * I used a interval checking (polling) */
		var body = editor.dom.doc.body;
		var thumbs = jQuery(body).find('img[src*="\\/thumb\\/phpThumb\\.php"]');
		
		if (thumbs.size() > 0) {
			thumbs.each(function(i) {
				/* get source image */
				var thumb_w = jQuery(this).width();
				var thumb_h = jQuery(this).height();
				
				var src = jQuery(this).attr('src');
				var pic_src;
				
				/* get picture source */
				var match = /\/thumb\/phpThumb\.php\?src=([^&]*)&?/i.exec(src);
				if (match != null) {
					pic_src = match[1];
				}
				
				/* apply picture source and dimensions */
				if (pic_src) {
					jQuery(this).attr('src', pic_src).attr('data-mce-src', pic_src).attr('_mce_src', pic_src);
					jQuery(this).attr('width', thumb_w).attr('data-mce-width', thumb_w).attr('_mce_width', thumb_w).width(thumb_w);
					jQuery(this).attr('height', thumb_h).attr('data-mce-height', thumb_h).attr('_mce_height', thumb_h).height(thumb_h);
				}			
			});
		}
	}, 800);
		
	/* set the click event on body, then limit by images */
	body.bind('click', function(ev) {
		var target = jQuery(ev.target);
		/* only process image clicks */
		if (target.is('img')) {
			impro.editor.showEditor(target);
		} else {
			jQuery('#impro_editor_box').addClass('closed');
		}		
		return true;
	});
};

/* this is triggered when the picture is resized */
impro.editor.onMCEResize = function(ed, img) {
	
};

/* show the editor and populate the fields */
impro.editor.showEditor = function(target) {
	
	var current_src = jQuery(target).attr('src');
	var current_domain = document.location.protocol + '//' + document.location.host;
	
	/* if the image begins with http://my.domain/, it was uploaded with classic wordpress upload. ask for migrating to image pro */
	if (current_src.indexOf(current_domain + '/') == 0) {
		if (confirm(imageproEditorL10N.convertToImagePro)) {
			var new_src = current_src.replace(current_domain, '');
			jQuery(target).attr('src', new_src).attr('data-mce-src', new_src).attr('_mce_src', new_src);
			impro.editor.showEditor(target);
		}
		return;
	}
	
	/* if does not begin with /, it means it's from an external domain */
	if (current_src[0] != "/") {
		impro.err(imageproEditorL10N.differentDomain);
		return;
	}
	
	/* open the editor panel */
	jQuery('#impro_editor_box').removeClass('closed');
	
	/* remember the clicked image (currently in editor) */
	impro.editor.$clickedImage = target;
	
	/* parse clicked image's path */
	var path = impro.editor.parseUrl(target.attr('src'));
	
	/* remember the old source */
	impro.editor.$clickedImage.get(0).old_src = target.attr('src');
	
	/* show thumb image */
	path['w'] = 50;
	path['h'] = 50;
	path['zc'] = 0;	
	var url = impro.editor.createThumbUrl(path);
	impro.editor.$editor.find('.thumb').attr('src', url);
	
	/* set alt and title */
	jQuery('.impro_alt').val(target.attr('alt'));
	jQuery('.impro_title').val(target.attr('title'));
	
	/* set align mode */
	var align = "alignnone";
	if (target.hasClass('alignleft')) {
		align = "alignleft";
	} else if (target.hasClass('aligncenter')) {
		align = "aligncenter";
	} else if (target.hasClass('alignright')) {
		align = "alignright";
	}
	jQuery('.impro_align').val(align);
	
	/* set link */
	if (impro.editor.$clickedImage.parent().is('a[href]')) {
		jQuery('.impro_link').attr('checked', 'checked');
	} else {
		jQuery('.impro_link').removeAttr('checked');
	}
};

/* change a given img element to what is contained in props */ 
impro.editor.resize = function(el, props) {
	if (!jQuery(el).is('img')) {	/* el must be an img tag */
		return null;
	}
	
	if (!(el instanceof jQuery)) {	/* make the element jQuery if not already */
		el = jQuery(el);
	}
	
	var el_dom = el.get(0);	/* also keep a DOM reference */
	
	var current_props = impro.editor.parseUrl(el.attr('src'));	/* get the current properties */
	
	/* if we could not extract properties from the source, get them from the image tag properties */
	if (!current_props['w']) {
		current_props['w'] = el_dom.width || el.width();
	}
	if (!current_props['h']) {
		current_props['h'] = el_dom.height || el.height();
	}
	
	/* copy properties */
	if (props['src']) {
		current_props['src'] = props['src'];
	}
	if (props['w']) {
		current_props['w'] = props['w'];
	}
	if (props['h']) {
		current_props['h'] = props['h'];
	}
	if (props['zc']) {
		current_props['zc'] = props['zc'];
	}
	
	var url = impro.editor.createThumbUrl(current_props);
	
	/* set preview */
	el.attr('src', props['src']);
	el.attr('width', props['w']);
	el.attr('height', props['h']);
	
	/* save */
	el.attr('_mce_src', props['src']);
	el.attr('_mce_width', props['w']);
	el.attr('_mce_height', props['h']);
	el.attr('data-mce-src', props['src']);
	el.attr('data-mce-width', props['w']);
	el.attr('data-mce-height', props['h']);
};

impro.editor.initEvents = function() {
	var changeProps = function() {
		/* preview */
		impro.editor.$clickedImage.attr('alt', jQuery('.impro_alt').val());
		impro.editor.$clickedImage.attr('title', jQuery('.impro_title').val());
		impro.editor.$clickedImage.removeClass('alignnone').removeClass('alignleft').removeClass('aligncenter').removeClass('alignright').addClass(jQuery('.impro_align').val());
		
		/* save */
		impro.editor.$clickedImage.attr('_mce_alt', jQuery('.impro_alt').val());
		impro.editor.$clickedImage.attr('_mce_title', jQuery('.impro_title').val());
		impro.editor.$clickedImage.attr('_mce_class', impro.editor.$clickedImage.attr('class'));
	};
	
	jQuery('.impro_align, .impro_alt, .impro_title').change(changeProps).keyup(changeProps);
	
	jQuery('.impro_link').click(function() {
		if (jQuery(this).is(':checked')) {
			tinyMCE.activeEditor.execCommand('mceInsertLink', false, { href: impro.editor.$clickedImage.attr('src'), target: '_blank' });
		} else {
			tinyMCE.activeEditor.execCommand('mceInsertLink', false, '');
		}		
	});
	
	jQuery('.impro_choose').click(function() {
		jQuery('#impro_folder_box #vimg-list').addClass('impro_choosebg');
		jQuery.scrollTo('#impro_folder_box',800);
		impro.editor.$chosen = impro.editor.$clickedImage;
	});
};

impro.editor.createThumbUrl = function(params) {
	var path = impro.url + '/src/thumb/phpThumb.php?';
	var tmp = [];
	for (var p in params) {
		if (['w','h','src','zc','q'].indexOf(p) === -1) {
			continue;
		}
		if (params.hasOwnProperty(p)) {
			tmp.push(p+'='+params[p]);
		}
	}
	return path + tmp.join('&');
}

/* fires when the editor is changed */
impro.editor.onSwitchEditor = function(id, mode) {
	
};

impro.editor.parseUrl = function(url) {
	var path = {};
	
	/* if begins with http(s?):// protocol */
	var is_http = /^https?:\/\/$/i.test(url);
	/* if contains phpThumb.php */
	var is_thumb = url.indexOf('/src/thumb/phpThumb.php') !== -1;
	
	if (is_thumb) {
		path['thumb'] = true;
		
		url.split('?')[1].split('&').forEach(function(v){
			var tmp = v.split('=');
			
			/* we have the width, height, quality and zoom-crop */
			if (tmp[0] === 'w' || tmp[0] === 'h' || tmp[0] === 'q' || tmp[0] === 'zc') {
				tmp[1] = parseInt(tmp[1], 10);
				/* not a numeric value, don't consider */
				if (isNaN(tmp[1])) {
					return;
				}
			}
			
			path[tmp[0]] = tmp[1];
		});
		
		path['url'] = url;
	} else {
		path['thumb'] = false;
		path['url'] = url;
		path['src'] = path['url'];
	}
	
	return path;
};

impro.editor.fakeImageResize = function () {
	var editor = tinyMCE.editors[0];
	var dom = editor.dom;
	var selection = editor.selection;
	var VK = tinyMCE.VK;
	var settings = editor.settings;
	
	var selectedElmX, selectedElmY, selectedElm, selectedElmGhost, selectedHandle, startX, startY, startW, startH, ratio,
		resizeHandles, width, height, rootDocument = document, editableDoc = editor.getDoc();

	if (settings.webkit_fake_resize === true) {
		alert('It seems that you are on a WebKit Browser (Chrome / Safari) '+
			  'and your configuration has webkit_fake_resize = true.' +
			  "\n\n" +
			  "Please turn it off for the Image Pro Plugin to function correctly under WebKit browsers.\n\n" + 
			  "Contact us for more information!");
		return;
	}
	
	/*if (!settings.object_resizing || settings.webkit_fake_resize === false) {
		return;
	}*/

	// Try disabling object resizing if WebKit implements resizing in the future
	try {
		editor.getDoc().execCommand("enableObjectResizing", false, false);
	} catch (ex) {
		// Ignore
	}

	// Details about each resize handle how to scale etc
	resizeHandles = {
		// Name: x multiplier, y multiplier, delta size x, delta size y
		n: [.5, 0, 0, -1],
		e: [1, .5, 1, 0],
		s: [.5, 1, 0, 1],
		w: [0, .5, -1, 0],
		nw: [0, 0, -1, -1],
		ne: [1, 0, 1, -1],
		se: [1, 1, 1, 1],
		sw : [0, 1, -1, 1]
	};

	function resizeElement(e) {
		var deltaX, deltaY;

		// Calc new width/height
		deltaX = e.screenX - startX;
		deltaY = e.screenY - startY;

		// Calc new size
		width = deltaX * selectedHandle[2] + startW;
		height = deltaY * selectedHandle[3] + startH;

		// Never scale down lower than 5 pixels
		width = width < 5 ? 5 : width;
		height = height < 5 ? 5 : height;

		// Constrain proportions when modifier key is pressed or if the nw, ne, sw, se corners are moved on an image
		if (VK.modifierPressed(e) || (selectedElm.nodeName == "IMG" && selectedHandle[2] * selectedHandle[3] !== 0)) {
			width = Math.round(height / ratio);
			height = Math.round(width * ratio);
		}

		// Update ghost size
		dom.setStyles(selectedElmGhost, {
			width: width,
			height: height
		});

		// Update ghost X position if needed
		if (selectedHandle[2] < 0 && selectedElmGhost.clientWidth <= width) {
			dom.setStyle(selectedElmGhost, 'left', selectedElmX + (startW - width));
		}

		// Update ghost Y position if needed
		if (selectedHandle[3] < 0 && selectedElmGhost.clientHeight <= height) {
			dom.setStyle(selectedElmGhost, 'top', selectedElmY + (startH - height));
		}
	}

	function endResize() {
		function setSizeProp(name, value) {
			if (value) {
				// Resize by using style or attribute
				/*if (selectedElm.style[name] || !editor.schema.isValid(selectedElm.nodeName.toLowerCase(), name)) {*/
				if (selectedElm.style[name] || !editor.schema.isValidChild(selectedElm.nodeName.toLowerCase(), name)) {
					dom.setStyle(selectedElm, name, value);
				} else {
					dom.setAttrib(selectedElm, name, value);
				}
			}
		}

		// Set width/height properties
		setSizeProp('width', width);
		setSizeProp('height', height);
		dom.setAttrib(selectedElm, 'data-mce-width', width);
		dom.setAttrib(selectedElm, 'data-mce-height', height);
		dom.setAttrib(selectedElm, 'width', width);
		dom.setAttrib(selectedElm, 'height', height);

		dom.unbind(editableDoc, 'mousemove', resizeElement);
		dom.unbind(editableDoc, 'mouseup', endResize);

		if (rootDocument != editableDoc) {
			dom.unbind(rootDocument, 'mousemove', resizeElement);
			dom.unbind(rootDocument, 'mouseup', endResize);
		}

		// Remove ghost and update resize handle positions
		dom.remove(selectedElmGhost);
		showResizeRect(selectedElm);
	}

	function showResizeRect(targetElm) {
		var position, targetWidth, targetHeight;

		hideResizeRect();

		// Get position and size of target
		position = dom.getPos(targetElm);
		selectedElmX = position.x;
		selectedElmY = position.y;
		targetWidth = targetElm.offsetWidth;
		targetHeight = targetElm.offsetHeight;

		// Reset width/height if user selects a new image/table
		if (selectedElm != targetElm) {
			selectedElm = targetElm;
			width = height = 0;
		}

		tinymce.each(resizeHandles, function(handle, name) {
			var handleElm;

			// Get existing or render resize handle
			handleElm = dom.get('mceResizeHandle' + name);
			if (!handleElm) {
				handleElm = dom.add(editableDoc.documentElement, 'div', {
					id: 'mceResizeHandle' + name,
					'class': 'mceResizeHandle',
					style: 'cursor:' + name + '-resize; margin:0; padding:0'
				});

				dom.bind(handleElm, 'mousedown', function(e) {
					e.preventDefault();

					endResize();

					startX = e.screenX;
					startY = e.screenY;
					startW = selectedElm.clientWidth;
					startH = selectedElm.clientHeight;
					ratio = startH / startW;
					selectedHandle = handle;

					selectedElmGhost = selectedElm.cloneNode(true);
					dom.addClass(selectedElmGhost, 'mceClonedResizable');
					dom.setStyles(selectedElmGhost, {
						left: selectedElmX,
						top: selectedElmY,
						margin: 0
					});

					editableDoc.documentElement.appendChild(selectedElmGhost);

					dom.bind(editableDoc, 'mousemove', resizeElement);
					dom.bind(editableDoc, 'mouseup', endResize);

					if (rootDocument != editableDoc) {
						dom.bind(rootDocument, 'mousemove', resizeElement);
						dom.bind(rootDocument, 'mouseup', endResize);
					}
				});
			} else {
				dom.show(handleElm);
			}

			// Position element
			dom.setStyles(handleElm, {
				left: (targetWidth * handle[0] + selectedElmX) - (handleElm.offsetWidth / 2),
				top: (targetHeight * handle[1] + selectedElmY) - (handleElm.offsetHeight / 2)
			});
		});

		// Only add resize rectangle on WebKit and only on images
		if (!tinymce.isOpera && selectedElm.nodeName == "IMG") {
			selectedElm.setAttribute('data-mce-selected', '1');
		}
	}

	function hideResizeRect() {
		if (selectedElm) {
			selectedElm.removeAttribute('data-mce-selected');
		}

		for (var name in resizeHandles) {
			dom.hide('mceResizeHandle' + name);
		}
	}

	// Add CSS for resize handles, cloned element and selected
	var css = '.mceResizeHandle {' +
		'position: absolute;' +
		'border: 1px solid black;' +
		'background: #FFF;' +
		'width: 5px;' +
		'height: 5px;' +
		'z-index: 10000' +
	'}' +
	'.mceResizeHandle:hover {' +
		'background: #000' +
	'}' +
	'img[data-mce-selected] {' +
		'outline: 1px solid black' +
	'}' +
	'img.mceClonedResizable, table.mceClonedResizable {' +
		'position: absolute;' +
		'outline: 1px dashed black;' +
		'opacity: .5;' +
		'z-index: 10000' +
	'}';
	
	var el = editor.getDoc().createElement('style');
	el.innerHTML = css;
	editor.getDoc().getElementsByTagName('head')[0].appendChild(el);

	function updateResizeRect() {
		var controlElm = dom.getParent(selection.getNode(), 'table,img');

		// Remove data-mce-selected from all elements since they might have been copied using Ctrl+c/v
		tinymce.each(dom.select('img[data-mce-selected]'), function(img) {
			img.removeAttribute('data-mce-selected');
		});

		if (controlElm) {
			showResizeRect(controlElm);
		} else {
			hideResizeRect();
		}
	}

	// Show/hide resize rect when image is selected
	editor.onNodeChange.add(updateResizeRect);

	// Fixes WebKit quirk where it returns IMG on getNode if caret is after last image in container
	dom.bind(editableDoc, 'selectionchange', updateResizeRect);

	// Remove the internal attribute when serializing the DOM
	editor.serializer.addAttributeFilter('data-mce-selected', function(nodes, name) {
		var i = nodes.length;

		while (i--) {
			nodes[i].attr(name, null);
		}
	});
};

jQuery(function() {
	impro.editor.init();
});